import { Component, OnInit } from '@angular/core';
import { PageBase } from 'src/app/core/base/page-base';
import { ToastService } from 'src/app/service/toast.service';
import { ConfirmationDialogService } from 'src/app/service/confirmation-dialog.service';
import { PermissionsEnum } from 'src/app/model/auth/permissionsEnum';
import { Group } from 'src/app/model/entities/group';
import { GroupsDataService } from 'src/app/modules/admin/groups/groups.data.service';
import { GroupsColumns } from 'src/app/modules/admin/groups/groups.columns';
import { AuthService } from 'src/app/service/auth.service';
import { ConfigService } from 'src/app/service/config.service';
import { FeatureFlagService } from 'src/app/service/feature-flag.service';

@Component({
  selector: 'app-groups',
  templateUrl: './groups.page.html',
  styleUrl: './groups.page.scss',
})
export class GroupsPage
  extends PageBase<Group, GroupsDataService, GroupsColumns>
  implements OnInit
{
  constructor(
    private toast: ToastService,
    private confirmation: ConfirmationDialogService,
    private auth: AuthService,
    private config: ConfigService,
    private features: FeatureFlagService
  ) {
    super(true);
  }

  async ngOnInit() {
    this.requiredPermissions = {
      read: (await this.features.get('PerUserSetup'))
        ? []
        : [PermissionsEnum.groups],
      create: (await this.features.get('PerUserSetup'))
        ? []
        : (await this.auth.hasAnyPermissionLazy(
              this.requiredPermissions.create ?? []
            ))
          ? (this.requiredPermissions.create ?? [])
          : [],
      update: (await this.features.get('PerUserSetup'))
        ? []
        : (await this.auth.hasAnyPermissionLazy(
              this.requiredPermissions.update ?? []
            ))
          ? (this.requiredPermissions.update ?? [])
          : [],
      delete: (await this.features.get('PerUserSetup'))
        ? []
        : (await this.auth.hasAnyPermissionLazy(
              this.requiredPermissions.delete ?? []
            ))
          ? (this.requiredPermissions.delete ?? [])
          : [],
      restore: (await this.features.get('PerUserSetup'))
        ? []
        : (await this.auth.hasAnyPermissionLazy(
              this.requiredPermissions.restore ?? []
            ))
          ? (this.requiredPermissions.restore ?? [])
          : [],
    };
  }

  load(silent?: boolean): void {
    if (!silent) this.loading = true;
    this.dataService
      .load(this.filter, this.sort, this.skip, this.take)
      .subscribe(result => {
        this.result = result;
        this.loading = false;
      });
  }

  delete(group: Group): void {
    this.confirmation.confirmDialog({
      header: 'dialog.delete.header',
      message: 'dialog.delete.message',
      accept: () => {
        this.loading = true;
        this.dataService.delete(group).subscribe(() => {
          this.toast.success('action.deleted');
          this.load();
        });
      },
      messageParams: { entity: group.name },
    });
  }

  restore(group: Group): void {
    this.confirmation.confirmDialog({
      header: 'dialog.restore.header',
      message: 'dialog.restore.message',
      accept: () => {
        this.loading = true;
        this.dataService.restore(group).subscribe(() => {
          this.toast.success('action.restored');
          this.load();
        });
      },
      messageParams: { entity: group.name },
    });
  }
}
